#changelist {
    border: none;
}
#tree {
    margin: 1em 0 1em 1em;
}
ul.jqtree-tree .jqtree-toggler {
    top: 10%;
}

#tree .edit {
    margin-left: 0.5em;
    vertical-align: middle;
}

.jqtree-tree .jqtree-loading > div .jqtree-title:after {
    content: url(spinner.gif);
    margin-left: 8px;
}

#tree.jqtree-loading:after {
    content: url(spinner.gif);
}